package com.hanbit.comm.ejb.session;

import java.rmi.RemoteException;

import java.util.Vector;

import javax.ejb.*;

import javax.naming.*;


public class LoginSBBean extends HanbitSessionBean {
    public User login(String userid, String passwd) throws RemoteException, 
                                                           UserNotFoundException, 
                                                           RoleNotFoundException, 
                                                           ProgramNotFoundException,
                                                           InvalidPasswdException{
        User user = getUser(userid);
        if(!passwd.equals(user.getPasswd())) throw new InvalidPasswdException();
        Role[] roles = getRoles(userid);

        for (int i = 0; i < roles.length; i++) {
            roles[i].programs = getPrograms(roles[i].getId());
        }

        user.roles = roles;

        return user;
    }

    private User getUser(String userid) throws UserNotFoundException, 
                                               RemoteException {
        try {
            Context ctx = new InitialContext();
            QuerySBHome home = (QuerySBHome) ctx.lookup("QuerySB");
            QuerySB query = home.create();

            Parameter para = new Parameter();
            para.setVOClass(new User());
            para.setStoredProcedure("pk_sysID.sp_getUsersByUserid");
            para.addArgument(userid);

            Vector vt = query.select(para);

            if ((vt == null) || vt.isEmpty()) {
                throw new UserNotFoundException();
            }

            return (User) vt.get(0);
        } catch (Exception e) {
            throw new RemoteException(e.getMessage());
        }
    }

    private Program[] getPrograms(String roleid) throws ProgramNotFoundException, 
                                                        RemoteException {
        try {
            Context ctx = new InitialContext();
            QuerySBHome home = (QuerySBHome) ctx.lookup("QuerySB");
            QuerySB query = home.create();

            Parameter para = new Parameter();
            para.setVOClass(new Program());
            para.setStoredProcedure("pk_program.sp_getProgramsByRoleid");
            para.addArgument(roleid);

            Vector vt = query.select(para);

            if ((vt == null) || vt.isEmpty()) {
                throw new ProgramNotFoundException();
            }

            Program[] programs = new Program[vt.size()];

            for (int i = 0; i < vt.size(); i++) {
                programs[i] = (Program) vt.get(i);
            }

            return programs;
        } catch (Exception e) {
            throw new RemoteException(e.getMessage());
        }
    }

    private Role[] getRoles(String userid) throws RoleNotFoundException, 
                                                  RemoteException {
        try {
            Context ctx = new InitialContext();
            QuerySBHome home = (QuerySBHome) ctx.lookup("QuerySB");
            QuerySB query = home.create();

            Parameter para = new Parameter();
            para.setVOClass(new Role());
            para.setStoredProcedure("pk_role.sp_getRolesByUserid");
            para.addArgument(userid);

            Vector vt = query.select(para);

            if ((vt == null) || vt.isEmpty()) {
                throw new RoleNotFoundException();
            }

            Role[] roles = new Role[vt.size()];

            for (int i = 0; i < vt.size(); i++) {
                roles[i] = (Role) vt.get(i);
            }

            return roles;
        } catch (Exception e) {
            throw new RemoteException(e.getMessage());
        }
    }
}
