package com.hanbit.comm.valueobject;

import java.lang.reflect.*;

import java.sql.*;

import java.util.Vector;


public class ValueObjectFactoryByResultSet extends ValueObjectFactory {
    public ValueObjectFactoryByResultSet(Object source, Object valueobject) {
        super(source, valueobject);
    }

    public boolean hasMoreElements() {
        if (source == null) {
            return false;
        }

        ResultSet rset = (ResultSet) source;

        try {
            if (rset.next()) {
                return true;
            } else {
                rset.close();
            }
        } catch (SQLException se) {
            se.printStackTrace();

            return false;
        }

        return false;
    }

    public Object nextElement() {
        ResultSet rset = (ResultSet) source;
        Object vo = null;

        try {
            ResultSetMetaData metaData = rset.getMetaData();
            vo = valueobject.getClass().newInstance();

            Method[] methods = valueobject.getClass().getMethods();
            int colCount = metaData.getColumnCount();
            int rowCount = 0;

            for (int i = 0; i < colCount; i++) {
                String colName = metaData.getColumnName(i + 1);
                Method method = getMethod(methods, colName);

                if (method != null) {
                    Class[] types = method.getParameterTypes();
                    String typeName = types[0].getName();

                    if (typeName.equals("java.lang.String")) {
                        Object[] args = { rset.getString(i + 1) };
                        method.invoke(vo, args);
                    } else if (typeName.equals("int") || 
                                   typeName.equals("java.lang.Integer")) {
                        Object[] args = { new Integer(rset.getInt(i + 1)) };
                        method.invoke(vo, args);
                    } else if (typeName.equals("double") || 
                                   typeName.equals("java.lang.Double")) {
                        Object[] args = { new Double(rset.getDouble(i + 1)) };
                        method.invoke(vo, args);
                    } else if (typeName.equals("float") || 
                                   typeName.equals("java.lang.Float")) {
                        Object[] args = { new Float(rset.getFloat(i + 1)) };
                        method.invoke(vo, args);
                    } else if (typeName.equals("long") || 
                                   typeName.equals("java.lang.Long")) {
                        Object[] args = { new Long(rset.getLong(i + 1)) };
                        method.invoke(vo, args);
                    } else if (typeName.equals("java.sql.Timestamp")) {
                        Object[] args = { rset.getTimestamp(i + 1) };
                        method.invoke(vo, args);
                    } else {
                        Object[] args = { rset.getString(i + 1) };
                        method.invoke(vo, args);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return vo;
    }

    private Method getMethod(Method[] methods, String name) {
        for (int i = 0; i < methods.length; i++) {
            Method method = methods[i];

            if (method.getName().equalsIgnoreCase("set" + name)) {
                return method;
            }
        }

        return null;
    }
}